package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.data.domain.DOSchema;
import cn.ibizlab.core.data.service.IDOSchemaService;
import cn.ibizlab.core.data.filter.DOSchemaSearchContext;
import cn.ibizlab.util.annotation.VersionCheck;

@Slf4j
@Api(tags = {"业务实体" })
@RestController("api-doschema")
@RequestMapping("")
public class DOSchemaResource {

    @Autowired
    public IDOSchemaService doschemaService;

    @Autowired
    @Lazy
    public DOSchemaMapping doschemaMapping;

    @ApiOperation(value = "新建业务实体", tags = {"业务实体" },  notes = "新建业务实体")
	@RequestMapping(method = RequestMethod.POST, value = "/doschemas")
    public ResponseEntity<DOSchemaDTO> create(@Validated @RequestBody DOSchemaDTO doschemadto) {
        DOSchema domain = doschemaMapping.toDomain(doschemadto);
		doschemaService.create(domain);
        DOSchemaDTO dto = doschemaMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "批量新建业务实体", tags = {"业务实体" },  notes = "批量新建业务实体")
	@RequestMapping(method = RequestMethod.POST, value = "/doschemas/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<DOSchemaDTO> doschemadtos) {
        doschemaService.createBatch(doschemaMapping.toDomain(doschemadtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "获取业务实体", tags = {"业务实体" },  notes = "获取业务实体")
	@RequestMapping(method = RequestMethod.GET, value = "/doschemas/{doschema_id}")
    public ResponseEntity<DOSchemaDTO> get(@PathVariable("doschema_id") String doschema_id) {
        DOSchema domain = doschemaService.get(doschema_id);
        DOSchemaDTO dto = doschemaMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "删除业务实体", tags = {"业务实体" },  notes = "删除业务实体")
	@RequestMapping(method = RequestMethod.DELETE, value = "/doschemas/{doschema_id}")
    public ResponseEntity<Boolean> remove(@PathVariable("doschema_id") String doschema_id) {
         return ResponseEntity.status(HttpStatus.OK).body(doschemaService.remove(doschema_id));
    }

    @ApiOperation(value = "批量删除业务实体", tags = {"业务实体" },  notes = "批量删除业务实体")
	@RequestMapping(method = RequestMethod.DELETE, value = "/doschemas/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        doschemaService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "更新业务实体", tags = {"业务实体" },  notes = "更新业务实体")
	@RequestMapping(method = RequestMethod.PUT, value = "/doschemas/{doschema_id}")
    public ResponseEntity<DOSchemaDTO> update(@PathVariable("doschema_id") String doschema_id, @RequestBody DOSchemaDTO doschemadto) {
		DOSchema domain  = doschemaMapping.toDomain(doschemadto);
        domain .setId(doschema_id);
		doschemaService.update(domain );
		DOSchemaDTO dto = doschemaMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "批量更新业务实体", tags = {"业务实体" },  notes = "批量更新业务实体")
	@RequestMapping(method = RequestMethod.PUT, value = "/doschemas/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<DOSchemaDTO> doschemadtos) {
        doschemaService.updateBatch(doschemaMapping.toDomain(doschemadtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "检查业务实体", tags = {"业务实体" },  notes = "检查业务实体")
	@RequestMapping(method = RequestMethod.POST, value = "/doschemas/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody DOSchemaDTO doschemadto) {
        return  ResponseEntity.status(HttpStatus.OK).body(doschemaService.checkKey(doschemaMapping.toDomain(doschemadto)));
    }

    @ApiOperation(value = "获取业务实体草稿", tags = {"业务实体" },  notes = "获取业务实体草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/doschemas/getdraft")
    public ResponseEntity<DOSchemaDTO> getDraft(DOSchemaDTO dto) {
        DOSchema domain = doschemaMapping.toDomain(dto);
        return ResponseEntity.status(HttpStatus.OK).body(doschemaMapping.toDto(doschemaService.getDraft(domain)));
    }

    @ApiOperation(value = "保存业务实体", tags = {"业务实体" },  notes = "保存业务实体")
	@RequestMapping(method = RequestMethod.POST, value = "/doschemas/save")
    public ResponseEntity<DOSchemaDTO> save(@RequestBody DOSchemaDTO doschemadto) {
        DOSchema domain = doschemaMapping.toDomain(doschemadto);
        doschemaService.save(domain);
        return ResponseEntity.status(HttpStatus.OK).body(doschemaMapping.toDto(domain));
    }

    @ApiOperation(value = "批量保存业务实体", tags = {"业务实体" },  notes = "批量保存业务实体")
	@RequestMapping(method = RequestMethod.POST, value = "/doschemas/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<DOSchemaDTO> doschemadtos) {
        doschemaService.saveBatch(doschemaMapping.toDomain(doschemadtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

	@ApiOperation(value = "获取数据集", tags = {"业务实体" } ,notes = "获取数据集")
    @RequestMapping(method= RequestMethod.GET , value="/doschemas/fetchdefault")
	public ResponseEntity<List<DOSchemaDTO>> fetchDefault(DOSchemaSearchContext context) {
        Page<DOSchema> domains = doschemaService.searchDefault(context) ;
        List<DOSchemaDTO> list = doschemaMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

	@ApiOperation(value = "查询数据集", tags = {"业务实体" } ,notes = "查询数据集")
    @RequestMapping(method= RequestMethod.POST , value="/doschemas/searchdefault")
	public ResponseEntity<Page<DOSchemaDTO>> searchDefault(@RequestBody DOSchemaSearchContext context) {
        Page<DOSchema> domains = doschemaService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(doschemaMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}



}

